'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unsave = exports.save = undefined;


// npm install --save 'name'

let save = exports.save = function () {
  var ref = _asyncToGenerator(function* (name, index, total) {
    yield (0, _progress2.default)('Installing', `npm install --save ${ name }`, name, index, total);
    // npm 3 we need to install peerDependencies as well
    yield installPeerDeps(name);
  });

  return function save(_x, _x2, _x3) {
    return ref.apply(this, arguments);
  };
}();

// npm uninstall --save 'name'


let unsave = exports.unsave = function () {
  var ref = _asyncToGenerator(function* (name, index, total) {
    try {
      yield (0, _progress2.default)('Uninstalling', 'npm uninstall --save ' + name, name, index, total);
    } catch (e) {
      // manual uninstall
      yield (0, _fns.rm)((0, _fns.p)((0, _opts2.default)('modulesDir'), name));
      yield _disk2.default.packageJSON.write(function (current, write) {
        delete current.dependencies[name];
        write(current);
      });
    }
  });

  return function unsave(_x4, _x5, _x6) {
    return ref.apply(this, arguments);
  };
}();

// TODO: only do this if npm 3 `exec('npm -v').charAt(0) == 3`

let installPeerDeps = function () {
  var ref = _asyncToGenerator(function* (name) {
    // instead of using npm view we just read package.json, safer
    const pkg = yield (0, _fns.readJSON)((0, _fns.p)((0, _opts2.default)('motionDir'), 'node_modules', name, 'package.json'));
    const peers = pkg.peerDependencies;

    // install peerdeps
    if (peers && typeof peers == 'object') {
      const peersArr = (0, _normalize2.default)(Object.keys(peers));

      if (peersArr.length) {
        print(`  Installing ${ name } peerDependencies`.bold);

        const peersFull = peersArr.map(function (name) {
          const version = latestVersion(name, peers[name]);

          if (!version) throw new Error(`No valid version range for package ${ name }@${ peers[name] }`);

          return `${ name }@${ version }`;
        });

        yield Promise.all(peersFull.map(function (full) {
          return (0, _execPromise2.default)(`npm install --save ${ full }`, (0, _opts2.default)('motionDir'));
        }));

        print('  ✓'.green, peersArr.join(', '));
        return peersArr;
      }
    }
  });

  return function installPeerDeps(_x7) {
    return ref.apply(this, arguments);
  };
}();

var _disk = require('../../disk');

var _disk2 = _interopRequireDefault(_disk);

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _fns = require('../../lib/fns');

var _execPromise = require('../../lib/execPromise');

var _execPromise2 = _interopRequireDefault(_execPromise);

var _normalize = require('./normalize');

var _normalize2 = _interopRequireDefault(_normalize);

var _progress = require('./progress');

var _progress2 = _interopRequireDefault(_progress);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function splitVersions(range) {
  const found = range.match(/[0-9\.]+/g);
  return found.length ? found : range;
}

function latestVersion(name, range) {
  try {
    return _semver2.default.maxSatisfying(splitVersions(range), range);
  } catch (e) {
    const simpleFindVer = range.replace(/[^0-9\. ]+/g, '').split(' ')[0];

    print(`  Error in semver of package ${ name } ${ e.message }`.yellow);
    print("  Attempting ", simpleFindVer);

    return simpleFindVer;
  }
}exports.default = {
  save, unsave
};
//# sourceMappingURL=npm.js.map