'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = log;

var _opts = require('../opts');

var _opts2 = _interopRequireDefault(_opts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

let debug = false;

const subIcons = {
  IN: ' ⇠ '.blue,
  OUT: ' ⇢ '.bold.green,
  SIN: ' ⇜ '.magenta,
  SOUT: ' ⇝ '.bold.yellow,
  RELOAD: ' ↺ '.yellow,
  STAR: ' ★ '.yellow,
  PIPE: ' | '.gray,
  CHECK: ' ✓ ',
  X: ' ✖ ',
  PLUS: ' ✚ ',
  ASTERISK: ' ✺ ',
  HAPPY: ' ☺ ',
  SAD: ' ☹ ',
  WRITE: ' ✍ ',
  DOWN: ' ↓ ',
  UP: ' ↑ '
};

function log(info, subIcon) {
  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  if (!debug) return;

  const isInfo = typeof info == 'object' && info.name && info.icon;

  let icon, subiout;

  if (isInfo) {
    const subi = subIcons[subIcon];
    subiout = subi ? subi : `    ${ subIcon }`;
    args = [info.icon, subiout].concat(_toConsumableArray(args));
  } else args = [info, subIcon].concat(_toConsumableArray(args));

  const doLog = () => print.apply(undefined, ['   '].concat(_toConsumableArray(colorArgs(args))));

  // all
  if (!debug.length) return doLog();
  // filtered
  if (info.name && debug.indexOf(info.name) >= 0) doLog();
}

log.externals = log.bind(null, { name: 'externals', icon: '🚀' });
log.internals = log.bind(null, { name: 'internals', icon: '🏠' });
log.cache = log.bind(null, { name: 'cache', icon: '💰' });
log.opts = log.bind(null, { name: 'opts', icon: '❍' });
log.gulp = log.bind(null, { name: 'gulp', icon: '👇' });
log.writer = log.bind(null, { name: 'writer', icon: '✎' });
log.file = log.bind(null, { name: 'file', icon: '▻' });
log.webpack = log.bind(null, { name: 'webpack', icon: '🎒' });
log.bridge = log.bind(null, { name: 'bridge', icon: '🚃' });

function colorArgs(args) {
  return args.map(arg => typeof arg === 'boolean' ? !!arg ? `${ arg }`.bold.green : `${ arg }`.bold.red : arg);
}

log.setLogging = function () {
  debug = (0, _opts2.default)('debug');
};

let timeName = null;
let startTime = null;

log.start = name => {
  timeName = name;
  startTime = Date.now();
};

log.end = () => {
  print('TIME', timeName, Date.now() - startTime);
};
//# sourceMappingURL=log.js.map